@@ -15,7 +15,10 @@ pub struct State<'a> {
     pub(crate) parent_clip_path: Option<svgtree::Node<'a>>,
     pub(crate) parent_marker: Option<svgtree::Node<'a>>,
     pub(crate) fe_image_link: bool,
+    /// The size of the root SVG element.
+    /// Right now, used only by use_node::get_clip_rect.
     pub(crate) size: Size,
+    /// A viewBox of the parent SVG element.
     pub(crate) view_box: Rect,
     pub(crate) opt: &'a OptionsRef<'a>,
 }
@@ -136,27 +139,7 @@ pub(crate) fn convert_doc(
     remove_unused_defs(&mut tree);
 
     if restore_viewbox {
-        let mut right = 0.0;
-        let mut bottom = 0.0;
-
-        for node in tree.root().descendants() {
-            if !tree.is_in_defs(&node) {
-                if let Some(bbox) = node.calculate_bbox() {
-                    if bbox.right() > right {
-                        right = bbox.right();
-                    }
-                    if bbox.bottom() > bottom {
-                        bottom = bbox.bottom();
-                    }
-                }
-            }
-        }
-
-        if let Some(rect) = Rect::new(0.0, 0.0, right, bottom) {
-            tree.set_view_box(rect);
-        }
-
-        tree.set_dimensions(right, bottom);
+        calculate_svg_bbox(&mut tree);
     }
 
     Ok(tree)
@@ -222,6 +205,33 @@ fn resolve_svg_size(
     (size.ok_or(Error::InvalidSize), restore_viewbox)
 }
 
+/// Calculates SVG's size and viewBox in case there were not set.
+///
+/// Simply iterates over all nodes and calculates a bounding box.
+fn calculate_svg_bbox(tree: &mut Tree) {
+    let mut right = 0.0;
+    let mut bottom = 0.0;
+
+    for node in tree.root().descendants() {
+        if !tree.is_in_defs(&node) {
+            if let Some(bbox) = node.calculate_bbox() {
+                if bbox.right() > right {
+                    right = bbox.right();
+                }
+                if bbox.bottom() > bottom {
+                    bottom = bbox.bottom();
+                }
+            }
+        }
+    }
+
+    if let Some(rect) = Rect::new(0.0, 0.0, right, bottom) {
+        tree.set_view_box(rect);
+    }
+
+    tree.set_dimensions(right, bottom);
+}
+
 #[inline(never)]
 pub(crate) fn convert_children(
     parent_node: svgtree::Node,
@@ -768,6 +778,10 @@ fn link_fe_image(
 }
 
 fn is_id_used(tree: & Tree, id: &str) -> bool {
+    if id.is_empty() {
+        return false;
+    }
+
     macro_rules! check_id {
         ($from:expr, $id:expr) => {
             if let Some(ref id) = $from {
