@@ -3,7 +3,8 @@
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 use crate::svgtree::{self, AId};
-use super::{Color, Kind, Opacity};
+use crate::{Color, Opacity, SvgColorExt};
+use super::Kind;
 
 /// A flood filter primitive.
 ///
@@ -22,8 +23,12 @@ pub struct Flood {
 }
 
 pub(crate) fn convert(fe: svgtree::Node) -> Kind {
+    let (color, opacity) = fe.attribute(AId::FloodColor)
+        .unwrap_or_else(svgtypes::Color::black)
+        .split_alpha();
+
     Kind::Flood(Flood {
-        color: fe.attribute(AId::FloodColor).unwrap_or_else(Color::black),
-        opacity: fe.attribute(AId::FloodOpacity).unwrap_or_default(),
+        color,
+        opacity: opacity * fe.attribute(AId::FloodOpacity).unwrap_or_default(),
     })
 }
