@@ -126,6 +126,7 @@ mod switch;
 mod units;
 mod use_node;
 
+pub use image::ImageHrefResolver;
 pub use svgtypes::{Align, AspectRatio};
 
 #[cfg(feature = "text")] pub use fontdb;
@@ -801,6 +802,11 @@ fn calc_node_bbox(
                 }
             }
 
+            // Make sure bbox was changed.
+            if bbox.fuzzy_eq(&PathBbox::new_bbox()) {
+                return None;
+            }
+
             Some(bbox)
         }
         _ => None,
